from enum import Enum

from bluer_ugv.README.ugvs.comparison.features.classes import (
    Feature,
    Feature_Enum,
    Feature_Comparison,
)


class UGV_Control(Feature_Enum):
    AI = 2
    RC = 1

    @property
    def as_str(self):
        return {
            UGV_Control.AI: "هوش مصنوعی",
            UGV_Control.RC: "رادیویی",
        }[self]


class ControlFeature(Feature):
    nickname = "control"
    long_name = "سامانه‌ی کنترلی"

    comparison_as_str = {
        Feature_Comparison.HIGHER: "هوش مصنوعی",
        Feature_Comparison.LOWER: "رادیویی",
        Feature_Comparison.SIMILAR: "مشابه",
    }



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import algo_docs
from bluer_ugv.README.swallow.consts import swallow_assets2
from bluer_ugv.README.swallow.digital.algo import navigation, yolo

docs = (
    [
        {
            "path": "../docs/swallow/digital/algo",
        },
        {
            "path": "../docs/swallow/digital/algo/driving.md",
        },
    ]
    + navigation.docs
    + [
        {
            "path": "../docs/swallow/digital/algo/tracking",
            "items": ImageItems(
                {
                    f"{swallow_assets2}/target-selection.png": f"{algo_docs}/socket.md",
                }
            ),
        }
    ]
    + yolo.docs
)



from bluer_ugv.README.ugvs.comparison.features.classes import Feature


class ConcealmentFeature(Feature):
    nickname = "concealment"
    long_name = "قابلیت استتار و کمین درازمدت"

    comparison_as_str = {}



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.ugvs.db import dict_of_ugvs
from bluer_ugv.README.validations.db import dict_of_validations

docs = [
    {
        "path": f"../docs/UGVs/{ugv_name}.md",
        "items": ImageItems({item: "" for item in info.get("items", [])}),
        "macros": {
            "validations:::": [
                "validations: {}".format(
                    ", ".join(
                        sorted(
                            [
                                f"[`{validation_name}`](../validations/{validation_name}.md)"
                                for validation_name, info in dict_of_validations.items()
                                if any(
                                    ugv_name_.startswith(f"{ugv_name}:")
                                    for ugv_name_ in info["ugv_name"]
                                )
                            ]
                        )
                    )
                ),
            ]
        },
    }
    for ugv_name, info in dict_of_ugvs.items()
]



from typing import Any

from bluer_ugv.README.ugvs.comparison.features.classes import (
    Feature,
    Feature_Comparison,
)


class PayloadFeature(Feature):
    nickname = "payload"
    long_name = "توان حمل بار"

    comparison_as_str = {
        Feature_Comparison.HIGHER: "بیشتر",
        Feature_Comparison.LOWER: "کمتر",
        Feature_Comparison.SIMILAR: "مشابه",
    }

    @property
    def score_as_str_(self) -> str:
        return f"{self.score} کیلوگرم"



from bluer_ugv.README.ugvs.comparison.features.classes import (
    Feature,
    Feature_Comparison,
    Feature_Enum,
)


class UGV_Cost(Feature_Enum):
    LOW = 100  # < 100 mT ~= $100
    MEDIUM = 10  # < 1 MT ~= $10k
    HIGH = 1  #

    @property
    def as_str(self):
        return {
            UGV_Cost.LOW: "کم",
            UGV_Cost.MEDIUM: "متوسط",
            UGV_Cost.HIGH: "زیاد",
        }[self]


class CostFeature(Feature):
    nickname = "cost"
    long_name = "هزینه"

    comparison_as_str = {
        Feature_Comparison.HIGHER: "پایین‌تر",
        Feature_Comparison.LOWER: "بالاتر",
        Feature_Comparison.SIMILAR: "مشابه",
    }



import os
from typing import List, Union
from functools import reduce

from blueness import module
from bluer_objects import file

from bluer_ugv import NAME
from bluer_ugv.README.ugvs.comparison.ugvs.db import list_of_ugvs
from bluer_ugv.README.ugvs.comparison.references.db import list_of_references
from bluer_ugv.logger import logger


NAME = module.name(__file__, NAME)


def build(
    log: bool = True,
    reference_ugv_name: str = "arzhang",
) -> bool:
    reference_ugv = list_of_ugvs.get(reference_ugv_name)
    if reference_ugv is None:
        logger.error(f"{reference_ugv_name}: reference ugv not found.")
        return False

    logger.info(f"{NAME}.build")

    success, contents = file.load_text(
        file.absolute(
            "../../../assets/comparison.html",
            reference_path=file.path(__file__),
        ),
        log=log,
    )
    if not success:
        return success

    try:
        content_index = [line.strip() for line in contents].index("contents:::")
    except Exception as e:
        logger.error(f"contents::: not found: {e}")
        return False

    contents = (
        contents[:content_index]
        + row_of(
            [
                "رديف",
                "نام محصول مشابه",
                "مشابهت و تفاوت ها محصول شما با آنها ",
            ],
            header=True,
        )
        + reduce(
            lambda x, y: x + y,
            [
                row_of(
                    [
                        str(index + 1),
                        ugv.description,
                        reference_ugv.compare(ugv),
                    ]
                )
                for index, ugv in enumerate(
                    [
                        ugv_
                        for ugv_ in reversed(
                            sorted(
                                list_of_ugvs.db,
                                key=lambda ugv: ugv.name,
                            )
                        )
                        if ugv_.nickname not in [reference_ugv_name, "template"]
                    ]
                )
            ],
            [],
        )
        + contents[content_index + 1 :]
        + ['<ol dir="rtl" style="text-align:right;">']
        + [
            '<li dir="rtl" style="text-align:{};"><a href="{}">{} - {}</a></li>'.format(
                "right" if reference.is_in_farsi else "left",
                reference.url,
                reference.title,
                reference.url,
            )
            for reference in reversed(
                sorted(
                    list_of_references.db,
                    key=lambda reference: reference.title,
                )
            )
            if reference.title != "template"
        ]
        + ['</ol dir="rtl" style="text-align:right;">']
    )

    return file.save_text(
        file.absolute(
            "../../../docs/UGVs/comparison.html",
            reference_path=file.path(__file__),
        ),
        contents,
        log=log,
    )


def row_of(
    row: Union[List[str], List[List[str]]],
    header: bool = False,
) -> List[str]:
    return (
        ["{}<tr>".format(8 * "")]
        + reduce(
            lambda x, y: x + y,
            [
                [
                    "{}<{}>".format(
                        12 * " ",
                        "th" if header else "td",
                    )
                ]
                + ([item] if isinstance(item, str) else list(item))
                + [
                    "</{}>".format(
                        "th" if header else "td",
                    )
                ]
                for item in row
            ],
            [],
        )
        + ["{}</tr>".format(8 * "")]
    )



from bluer_ugv.README.ugvs.comparison.references.classes import List_of_References

list_of_references = List_of_References()

list_of_references.add(
    title="ربات موشک انداز نذیر",
    url="https://fa.wikipedia.org/wiki/%D9%86%D8%B0%DB%8C%D8%B1_(%D8%B1%D8%A8%D8%A7%D8%AA_%D9%85%D9%88%D8%B4%DA%A9%E2%80%8C%D8%A7%D9%86%D8%AF%D8%A7%D8%B2)",
    list_of_ugvs=[
        "nazir",
    ],
    is_in_farsi=True,
)

list_of_references.add(
    title="ربات ایرانی که تانک‌های دشمن را شکار می‌کند",
    url="https://www.mashreghnews.ir/news/1746417/%D8%B1%D8%A8%D8%A7%D8%AA-%D8%A7%DB%8C%D8%B1%D8%A7%D9%86%DB%8C-%DA%A9%D9%87-%D8%AA%D8%A7%D9%86%DA%A9-%D9%87%D8%A7%DB%8C-%D8%AF%D8%B4%D9%85%D9%86-%D8%B1%D8%A7-%D8%B4%DA%A9%D8%A7%D8%B1-%D9%85%DB%8C-%DA%A9%D9%86%D8%AF-%D9%81%DB%8C%D9%84%D9%85",
    list_of_ugvs=[
        "heydar",
    ],
    is_in_farsi=True,
)

list_of_references.add(
    title="ربات‌هایی که بلای جان تانک‌های دشمن می‌شوند",
    url="https://farsnews.ir/M_r7726/1738414889748074000/%D8%B1%D8%A8%D8%A7%D8%AA%D9%87%D8%A7%DB%8C%DB%8C-%DA%A9%D9%87-%D8%A8%D9%84%D8%A7%DB%8C-%D8%AC%D8%A7%D9%86-%D8%AA%D8%A7%D9%86%DA%A9%D9%87%D8%A7%DB%8C-%D8%AF%D8%B4%D9%85%D9%86-%D9%85%DB%8C%D8%B4%D9%88%D9%86%D8%AF",
    list_of_ugvs=[
        "nazir",
        "heydar",
        "karakal",
    ],
    is_in_farsi=True,
)


list_of_references.add(
    title="سپاه با لشکری از «قاسم» و «یونس» به جمع ارتش‌های رباتیک جهان پیوست",
    url="https://www.mashreghnews.ir/news/1275493/%D8%B3%D9%BE%D8%A7%D9%87-%D8%A8%D8%A7-%D9%84%D8%B4%DA%A9%D8%B1%DB%8C-%D8%A7%D8%B2-%D9%82%D8%A7%D8%B3%D9%85-%D9%88-%DB%8C%D9%88%D9%86%D8%B3-%D8%A8%D9%87-%D8%AC%D9%85%D8%B9-%D8%A7%D8%B1%D8%AA%D8%B4-%D9%87%D8%A7%DB%8C-%D8%B1%D8%A8%D8%A7%D8%AA%DB%8C%DA%A9-%D8%AC%D9%87%D8%A7%D9%86",
    list_of_ugvs=[
        "qasem",
        "raad1",
    ],
    is_in_farsi=True,
)

list_of_references.add(
    title="ربات‌های جنگی در ایران عملیاتی شدند",
    url="https://snn.ir/fa/news/1249403/%D8%B1%D8%A8%D8%A7%D8%AA%E2%80%8C%D9%87%D8%A7%DB%8C-%D8%AC%D9%86%DA%AF%DB%8C-%D8%AF%D8%B1-%D8%A7%DB%8C%D8%B1%D8%A7%D9%86-%D8%B9%D9%85%D9%84%DB%8C%D8%A7%D8%AA%DB%8C-%D8%B4%D8%AF%D9%86%D8%AF-%D9%85%D8%B9%D8%B1%D9%81%DB%8C-%DB%B5-%D8%B1%D8%A8%D8%A7%D8%AA-%D8%AC%D9%86%DA%AF%DB%8C-%D8%B3%D8%A7%D8%AE%D8%AA-%D8%A7%DB%8C%D8%B1%D8%A7%D9%86",
    list_of_ugvs=[
        "karakal",
        "nazir",
        "qasem",
        "heydar",
        "sepah",
    ],
    is_in_farsi=True,
)

list_of_references.add(
    title="با این سلاح جدید ایرانی بیشتر آشنا شوید",
    url="https://www.hamshahrionline.ir/news/978710/%D9%87%D9%85%D9%87-%DA%86%DB%8C%D8%B2-%D8%AF%D8%B1%D8%A8%D8%A7%D8%B1%D9%87-%D8%A2%D8%B1%DB%8C%D8%A7-%D8%A7%D8%B2-%D8%AA%D8%B3%D9%84%DB%8C%D8%AD-%D8%A8%D9%87-%D9%85%D8%B3%D9%84%D8%B3%D9%84-%D8%AA%D8%A7-%D9%82%D8%AF%D8%B1%D8%AA-%D9%86%D8%A8%D8%B1%D8%AF-%D9%87%D9%88%D8%B4%D9%85%D9%86%D8%AF",
    list_of_ugvs=[
        "arya",
    ],
    is_in_farsi=True,
)

list_of_references.add(
    title="رونمایی از ربات جنگنده آریا ",
    url="https://www.ilna.ir/%D8%A8%D8%AE%D8%B4-%D8%B3%D8%A7%DB%8C%D8%B1-%D8%B1%D8%B3%D8%A7%D9%86%D9%87-%D9%87%D8%A7-10/1686715-%D8%B1%D9%88%D9%86%D9%85%D8%A7%DB%8C%DB%8C-%D8%A7%D8%B2-%D8%B1%D8%A8%D8%A7%D8%AA-%D8%AC%D9%86%DA%AF%D9%86%D8%AF%D9%87-%D8%A2%D8%B1%DB%8C%D8%A7-%D9%81%DB%8C%D9%84%D9%85",
    list_of_ugvs=[
        "arya",
    ],
    is_in_farsi=True,
)

list_of_references.add(
    title="Iran Aria combat robot",
    url="https://en.wikipedia.org/wiki/Iran_Aria_combat_robot",
    list_of_ugvs=[
        "arya",
    ],
)

list_of_references.add(
    title="Iran Heydar combat robot",
    url="https://en.wikipedia.org/wiki/Iran_Heydar_combat_robot",
    list_of_ugvs=[
        "heydar",
    ],
)

list_of_references.add(
    title="THeMIS",
    url="https://en.wikipedia.org/wiki/THeMIS",
    list_of_ugvs=[
        "themis_9",
    ],
)

list_of_references.add(
    title="Type-X (unmanned ground vehicle)",
    url="https://en.wikipedia.org/wiki/Type-X_(unmanned_ground_vehicle)",
    list_of_ugvs=[
        "type_x",
    ],
)

list_of_references.add(
    title="Uran-6",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/uran-6-ugv-mrtk-r",
    list_of_ugvs=[
        "uran_6",
    ],
)

list_of_references.add(
    title="Uran-9 UGV",
    url="https://en.wikipedia.org/wiki/Uran-9",
    list_of_ugvs=[
        "uran_9",
    ],
)

list_of_references.add(
    title="Uran-9",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/uran-9-ugcv-ugv-unmanned-ground-combat-vehicle-technical-data-sheet-specifications-pictures-video-10910163",
    list_of_ugvs=[
        "uran_6",
    ],
)

list_of_references.add(
    title="Uran-14 MRTK-P",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/uran-14-mrtk-p-unmanned-robotic-mine-clearance-and-firefighting-tracked-vehicle-data-sheet-specifications-pictures-video11201166",
    list_of_ugvs=[
        "uran_14",
    ],
)

list_of_references.add(
    title="Centaur Unmanned Ground Vehicle",
    url="https://www.army-technology.com/projects/centaur-unmanned-ground-vehicle/",
    list_of_ugvs=[
        "centaur",
    ],
)

list_of_references.add(
    title="XM1219 armed robotic vehicle",
    url="https://en.wikipedia.org/wiki/XM1219_armed_robotic_vehicle",
    list_of_ugvs=[
        "xm1219",
    ],
)


list_of_references.add(
    title="Foster-Miller TALON",
    url="https://en.wikipedia.org/wiki/Foster-Miller_TALON",
    list_of_ugvs=[
        "talon",
    ],
)

list_of_references.add(
    title="Gladiator tactical unmanned ground vehicle",
    url="https://en.wikipedia.org/wiki/Gladiator_tactical_unmanned_ground_vehicle",
    list_of_ugvs=[
        "gladiator",
    ],
)


list_of_references.add(
    title="Designation: UKAP",
    url="https://www.army-guide.com/eng/product5766.html",
    list_of_ugvs=[
        "ukap",
    ],
)

list_of_references.add(
    title="Ripsaw (vehicle)",
    url="https://en.wikipedia.org/wiki/Ripsaw_(vehicle)",
    list_of_ugvs=[
        "ripsaw",
    ],
)

list_of_references.add(
    title="tEODor Evo",
    url="https://www.avinc.com/ugv/teodor-evo",
    list_of_ugvs=[
        "teodor",
    ],
)

list_of_references.add(
    title="Black Knight (vehicle)",
    url="https://en.wikipedia.org/wiki/Black_Knight_(vehicle)",
    list_of_ugvs=[
        "black_knight",
    ],
)

list_of_references.add(
    title="BAE Systems Black Knight Tank - Unmanned Combat Vehicle",
    url="https://militarymachine.com/black-knight-tank/",
    list_of_ugvs=[
        "black_knight",
    ],
)

list_of_references.add(
    title="Miloš (unmanned ground vehicle)",
    url="https://en.wikipedia.org/wiki/Milo%C5%A1_(unmanned_ground_vehicle)",
    list_of_ugvs=[
        "milica",
    ],
)

list_of_references.add(
    title="Milca UGV",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/milca-serbia",
    list_of_ugvs=[
        "milica",
    ],
)

list_of_references.add(
    title="Taifun-M anti-commando vehicle fully operational in Russian Strategic Missile Forces TASS 11601171.",
    url="https://www.armyrecognition.com/archives/archives-land-defense/2017/taifun-m-anti-commando-vehicle-fully-operational-in-russian-strategic-missile-forces-tass-11601171",
    list_of_ugvs=[
        "taifun_m",
    ],
)

list_of_references.add(
    title="Jackal: Unmanned Ground Vehicle",
    url="https://clearpathrobotics.com/jackal-small-unmanned-ground-vehicle/",
    list_of_ugvs=[
        "jackal",
    ],
)


list_of_references.add(
    title="AvantGuard UGCV",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/avantguard-israel-uk",
    list_of_ugvs=[
        "avantguard",
    ],
)


list_of_references.add(
    title="AvantGuard",
    url="https://en.wikipedia.org/wiki/AvantGuard",
    list_of_ugvs=[
        "avantguard",
    ],
)


list_of_references.add(
    title="SIM RACAR UGV",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/sim-racar-lambda-germany-uk",
    list_of_ugvs=[
        "sr-lambda",
    ],
)


list_of_references.add(
    title="SIM-RACAR-Lambda",
    url="https://www.army-guide.com/eng/product5740.html",
    list_of_ugvs=[
        "sr-lambda",
    ],
)

list_of_references.add(
    title="Sharp Claw 2",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/sharp-claw-2-ugv-6x6-unmanned-ground-vehicle-technical-data-sheet-specifications-pictures-video-11412165",
    list_of_ugvs=[
        "sharp_claw_2",
    ],
)

list_of_references.add(
    title="Sharp Claw 1",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/sharp-claw-1-ugv-unmanned-ground-vehicle-china-data-fact-sheet",
    list_of_ugvs=[
        "sharp_claw_1",
    ],
)

list_of_references.add(
    title="Robattle",
    url="https://en.wikipedia.org/wiki/Robattle",
    list_of_ugvs=[
        "robattle",
    ],
)


list_of_references.add(
    title="Guardium",
    url="https://en.wikipedia.org/wiki/Guardium",
    list_of_ugvs=[
        "guardium",
    ],
)


list_of_references.add(
    title="Guardium UGV",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/guardium-ugv-israel-uk",
    list_of_ugvs=[
        "guardium",
    ],
)

list_of_references.add(
    title="Guardium-LS",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles/guardium-lg-israel-uk",
    list_of_ugvs=[
        "guardium",
    ],
)

list_of_references.add(
    title="Unmanned Ground Vehicles",
    url="https://www.armyrecognition.com/military-products/army/unmanned-systems/unmanned-ground-vehicles",
)


list_of_references.add(
    title="template",
    url="template",
    list_of_ugvs=[
        "template",
    ],
)



from bluer_ugv.README.swallow.consts import swallow_assets2
from bluer_ugv.README.arzhang.consts import arzhang_assets, arzhang_assets2
from bluer_ugv.README.rangin.consts import rangin_assets2

dict_of_ugvs = {
    "swallow": {
        "items": [
            f"{swallow_assets2}/20250701_2206342_1.gif",
            f"{swallow_assets2}/20250913_203635~2_1.gif",
        ]
    },
    "arzhang": {
        "items": [
            f"{arzhang_assets2}/20251106_111750.jpg",
        ],
    },
    "arzhang2": {
        "items": [
            f"{arzhang_assets2}/20251106_111750.jpg",
        ],
    },
    "arzhang3": {
        "items": [
            f"{arzhang_assets}/20251107_175506.jpg",
        ],
    },
    "rangin": {
        "items": [
            f"{rangin_assets2}/rangin.png",
        ]
    },
}



from typing import List
import threading

from bluer_options.timer import Timer
from bluer_options import string
from bluer_objects.storage.policies import DownloadPolicy
from bluer_objects import storage
from bluer_objects.metadata import post_to_object, get_from_object
from bluer_sbc.imager.camera import instance as camera
from bluer_sbc.env import BLUER_SBC_CAMERA_WIDTH
from bluer_algo.yolo.dataset.classes import YoloDataset
from bluer_algo.yolo.model.predictor import YoloPredictor
from bluer_algo.socket.message import SocketMessage

from bluer_ugv import env
from bluer_ugv.swallow.session.classical.camera.generic import ClassicalCamera
from bluer_ugv.swallow.session.classical.keyboard.classes import ClassicalKeyboard
from bluer_ugv.swallow.session.classical.leds import ClassicalLeds
from bluer_ugv.swallow.session.classical.setpoint.classes import ClassicalSetPoint
from bluer_ugv.swallow.session.classical.mode import OperationMode
from bluer_ugv.logger import logger


class ClassicalYoloCamera(ClassicalCamera):
    def __init__(
        self,
        keyboard: ClassicalKeyboard,
        leds: ClassicalLeds,
        setpoint: ClassicalSetPoint,
        object_name: str,
    ):
        super().__init__(keyboard, leds, setpoint, object_name)

        self.prediction_timer = Timer(
            period=env.BLUER_UGV_CAMERA_ACTION_PERIOD,
            name="{}.prediction".format(self.__class__.__name__),
            log=True,
        )
        self.training_timer = Timer(
            period=env.BLUER_UGV_CAMERA_TRAINING_PERIOD,
            name="{}.training".format(self.__class__.__name__),
            log=True,
        )

        self.dataset = YoloDataset(
            object_name=self.object_name,
            create=True,
        )

        assert super().initialize()

        if not storage.download(
            env.BLUER_UGV_SWALLOW_YOLO_MODEL,
            policy=DownloadPolicy.DOESNT_EXIST,
        ):
            logger.error("cannot download the model.")

        success, self.predictor = YoloPredictor.load(
            object_name=env.BLUER_UGV_SWALLOW_YOLO_MODEL,
            image_size=BLUER_SBC_CAMERA_WIDTH,
        )
        if not success:
            logger.error("cannot create the predictor.")

        self.running = True
        self.thread = threading.Thread(target=self.loop, daemon=True)
        self.thread.start()

    # the parent method is called in the constructor
    def initialize(self) -> bool:
        return True

    def cleanup(self):
        return True

    # the parent method is called in stop()
    def stop(self):
        self.running = False
        self.thread.join()

        logger.info(f"{self.__class__.__name__}.stopped.")

        super().cleanup()

        if not self.dataset.save(
            verbose=True,
        ):
            logger.error("cannot save the dataset.")

        if self.dataset.empty:
            return

        dataset_list: List[str] = get_from_object(
            object_name=env.BLUER_UGV_SWALLOW_YOLO_DATASET_LIST,
            key="dataset-list",
            default=[],
            download=True,
        )
        dataset_list.append(self.object_name)
        if not post_to_object(
            object_name=env.BLUER_UGV_SWALLOW_YOLO_DATASET_LIST,
            key="dataset-list",
            value=dataset_list,
            upload=True,
            verbose=True,
        ):
            logger.error("failed to add object to dataset list.")

    def loop(self):
        logger.info(f"{self.__class__.__name__}.loop started.")

        while self.running:
            mode = self.keyboard.get("mode", OperationMode.NONE)

            success = True
            if mode == OperationMode.ACTION:
                success = self.loop_action()

            if mode == OperationMode.TRAINING:
                success = self.loop_training()

            if not success:
                logger.error(f"loop failed, mode: {mode.name.lower()}.")

    def loop_action(self) -> bool:
        if not self.prediction_timer.tick():
            return True

        self.leds.flash("red")

        success, image = camera.capture(
            close_after=False,
            open_before=False,
            log=True,
        )
        if not success:
            return success

        debug_mode = self.keyboard.get("debug_mode", False)
        success, metadata = self.predictor.predict(
            image=image,
            return_annotated_image=debug_mode,
            annotated_image_scale=2,
        )
        if not success:
            return success

        if debug_mode:
            if not self.send_debug_data(
                SocketMessage(
                    {
                        "image": metadata["annotated_image"],
                    }
                )
            ):
                logger.warning("failed to send debug data.")

        if not metadata["detections"]:
            logger.info("no detections.")
            return True

        detection = metadata["detections"][0]
        logger.info("confidence: {:.2f}".format(detection["confidence"]))
        detection_x_center = (detection["bbox_xyxy"][0] + detection["bbox_xyxy"][2]) / 2
        if detection_x_center < image.shape[1] / 2:
            self.setpoint.put(
                what="steering",
                value=env.BLUER_UGV_SWALLOW_STEERING_SETPOINT,
                log=True,
                steering_expires_in=env.BLUER_UGV_SWALLOW_STEERING_YOLO_EXPIRY,
            )
        else:
            self.setpoint.put(
                what="steering",
                value=-env.BLUER_UGV_SWALLOW_STEERING_SETPOINT,
                log=True,
                steering_expires_in=env.BLUER_UGV_SWALLOW_STEERING_YOLO_EXPIRY,
            )

        return True

    def loop_training(self) -> bool:
        if not (self.training_timer.tick() or self.keyboard.last_key != ""):
            return True

        self.leds.flash("red")

        filename = "{}.png".format(
            string.pretty_date(
                as_filename=True,
                unique=True,
            )
        )

        success, _ = camera.capture(
            close_after=False,
            open_before=False,
            object_name=self.object_name,
            filename=filename,
            log=True,
        )
        if not success:
            return success

        # TODO: dataset +=

        self.training_timer.reset()

        return True



from enum import Enum

from bluer_ugv.README.ugvs.comparison.features.classes import (
    Feature,
    Feature_Comparison,
    Feature_Enum,
)


class UGV_Size(Feature_Enum):
    SMALL = 100  # < 5 kg
    MEDIUM = 10  # < 50 kg
    LARGE = 1

    @property
    def as_str(self):
        return {
            UGV_Size.SMALL: "کوچک",
            UGV_Size.MEDIUM: "متوسط",
            UGV_Size.LARGE: "بزرگ",
        }[self]


class SizeFeature(Feature):
    nickname = "size"
    long_name = "اندازه"

    comparison_as_str = {
        Feature_Comparison.HIGHER: "کوچکتر",
        Feature_Comparison.LOWER: "بزرگتر",
        Feature_Comparison.SIMILAR: "مشابه",
    }



import keyboard

from blueness import module

from bluer_ugv import NAME
from bluer_ugv.logger import logger

NAME = module.name(__file__, NAME)


def test(list_of_keys: str = "") -> bool:
    logger.info("{}.testing({}): ^C to stop".format(NAME, list_of_keys))

    try:
        while True:
            for key in list_of_keys:
                if keyboard.is_pressed(key):
                    logger.info(f"{key} is pressed.")
    except KeyboardInterrupt:
        logger.info("^C detected.")

    return True



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.rangin.consts import rangin_assets2


items = ImageItems(
    {
        f"{rangin_assets2}/concepts/11.png": "",
        f"{rangin_assets2}/concepts/12.png": "",
    }
)



import os

from bluer_options.help.functions import get_help
from bluer_objects import file, README

from bluer_ugv import NAME, VERSION, ICON, REPO_NAME
from bluer_ugv.help.functions import help_functions
from bluer_ugv.README import (
    alias,
    beast,
    eagle,
    fire,
    ravin,
    root,
    shield,
    arzhang,
    rangin,
    releases,
    swallow,
)
from bluer_ugv.README.ugvs import docs as ugvs
from bluer_ugv.README.ugvs.comparison.build import build as build_comparison


def build() -> bool:
    return (
        all(
            README.build(
                items=readme.get("items", []),
                path=os.path.join(file.path(__file__), readme["path"]),
                cols=readme.get("cols", 3),
                ICON=ICON,
                NAME=NAME,
                VERSION=VERSION,
                REPO_NAME=REPO_NAME,
                help_function=lambda tokens: get_help(
                    tokens,
                    help_functions,
                    mono=True,
                ),
                macros=readme.get("macros", {}),
            )
            for readme in root.docs
            + alias.docs
            + arzhang.docs
            + beast.docs
            + eagle.docs
            + fire.docs
            + rangin.docs
            + ravin.docs
            + releases.docs
            + shield.docs
            + ugvs.docs
            + swallow.docs
        )
        and build_comparison()
    )



from bluer_ugv.README.ravin import ravin3, ravin4
from bluer_ugv.README.ravin.items import items

docs = (
    [
        {
            "items": items,
            "path": "../docs/ravin",
        },
    ]
    + ravin3.docs
    + ravin4.docs
)



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.swallow.consts import (
    swallow_assets2,
    swallow_designs,
    swallow_electrical_designs,
)

items = ImageItems(
    {
        f"{swallow_electrical_designs}/digital.png": f"{swallow_electrical_designs}/digital.svg",
        f"{swallow_assets2}/20250609_164433.jpg": "",
        f"{swallow_assets2}/20250614_102301.jpg": "",
        f"{swallow_assets2}/20250614_114954.jpg": "",
        f"{swallow_assets2}/20250615_192339.jpg": "",
        f"{swallow_assets2}/20250703_153834.jpg": "",
        f"{swallow_assets2}/design/v2/01.jpg": "",
        f"{swallow_assets2}/design/v3/01.jpg": "",
        f"{swallow_assets2}/design/v4/01.jpg": "",
        f"{swallow_assets2}/20250925_213013.jpg": "",
        f"{swallow_assets2}/20250925_214017.jpg": "",
        f"{swallow_assets2}/20250928_160425.jpg": "",
        f"{swallow_assets2}/20250928_160449.jpg": "",
        f"{swallow_assets2}/design/head-v1/01.jpg": "",
        f"{swallow_assets2}/20251002_103712.jpg": "",
        f"{swallow_assets2}/20251002_103720.jpg": "",
        f"{swallow_assets2}/design/v5/01.jpg": "",
        f"{swallow_electrical_designs}/nuts-bolts-spacers.png": f"{swallow_electrical_designs}/nuts-bolts-spacers.svg",
        f"{swallow_assets2}/20251018_133202.jpg": "",
        f"{swallow_assets2}/20251018_133349.jpg": "",
        f"{swallow_assets2}/20251008_114557.jpg": "",
        f"{swallow_assets2}/20251008_133418.jpg": "",
        f"{swallow_assets2}/20251008_124129.jpg": "",
        f"{swallow_assets2}/20251008_124932.jpg": "",
        f"{swallow_designs}/kicad/swallow/exports/swallow.png": f"{swallow_designs}/kicad/swallow/exports/swallow.pdf",
        f"{swallow_designs}/kicad/swallow/exports/swallow-3d.png": "",
        f"{swallow_designs}/kicad/swallow/exports/swallow-3d-back.png": "",
        f"{swallow_designs}/kicad/swallow/exports/swallow-pcb.png": "",
        f"{swallow_assets2}/20250609_164433.jpg": "",
        f"{swallow_assets2}/20251112_085331.jpg": "",
        f"{swallow_assets2}/20251112_181047.jpg": "",
        f"{swallow_assets2}/20251112_181053.jpg": "",
    }
)


docs = [
    {
        "path": "../docs/swallow/digital/design/shield.md",
        "items": items,
    },
]
